@@ -597,8 +597,12 @@ void KQuickStyleItem::initStyleOption()
         QStyleOptionSlider *opt = qstyleoption_cast<QStyleOptionSlider*>(m_styleoption);
         opt->orientation = horizontal() ? Qt::Horizontal : Qt::Vertical;
         opt->upsideDown = !horizontal();
-        opt->minimum = qMax(0, minimum());
-        opt->maximum = qMax(0, maximum());
+
+        int min = minimum();
+        int max = std::max(min, maximum());
+
+        opt->minimum = min;
+        opt->maximum = max;
         opt->sliderPosition = value();
         opt->singleStep = step();
 
@@ -793,6 +797,54 @@ void KQuickStyleItem::resolvePalette()
     }
 }
 
+int KQuickStyleItem::leftPadding() const
+{
+    switch (m_itemType) {
+    case Frame: {
+        const QRect cr = KQuickStyleItem::style()->subElementRect(QStyle::SE_ShapedFrameContents, m_styleoption);
+        return cr.left() - m_styleoption->rect.left();
+    }
+    default:
+        return 0;
+    }
+}
+
+int KQuickStyleItem::topPadding() const
+{
+    switch (m_itemType) {
+    case Frame: {
+        const QRect cr = KQuickStyleItem::style()->subElementRect(QStyle::SE_ShapedFrameContents, m_styleoption);
+        return cr.top() - m_styleoption->rect.top();
+    }
+    default:
+        return 0;
+    }
+}
+
+int KQuickStyleItem::rightPadding() const
+{
+    switch (m_itemType) {
+    case Frame: {
+        const QRect cr = KQuickStyleItem::style()->subElementRect(QStyle::SE_ShapedFrameContents, m_styleoption);
+        return m_styleoption->rect.right() - cr.right();
+    }
+    default:
+        return 0;
+    }
+}
+
+int KQuickStyleItem::bottomPadding() const
+{
+    switch (m_itemType) {
+    case Frame: {
+        const QRect cr = KQuickStyleItem::style()->subElementRect(QStyle::SE_ShapedFrameContents, m_styleoption);
+        return m_styleoption->rect.bottom() - cr.bottom();
+    }
+    default:
+        return 0;
+    }
+}
+
 /*
  *   Property style
  *
@@ -896,7 +948,7 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
         }
         if (btn->toolButtonStyle != Qt::ToolButtonIconOnly) {
             QSize textSize = btn->fontMetrics.size(Qt::TextShowMnemonic, btn->text);
-            textSize.setWidth(textSize.width() + btn->fontMetrics.width(QLatin1Char(' '))*2);
+            textSize.setWidth(textSize.width() + btn->fontMetrics.horizontalAdvance(QLatin1Char(' '))*2);
             if (btn->toolButtonStyle == Qt::ToolButtonTextUnderIcon) {
                 h += 4 + textSize.height();
                 if (textSize.width() > w)
@@ -916,7 +968,7 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
     case Button: {
         QStyleOptionButton *btn = qstyleoption_cast<QStyleOptionButton*>(m_styleoption);
 
-        int contentWidth = btn->fontMetrics.width(btn->text);
+        int contentWidth = btn->fontMetrics.boundingRect(btn->text).width();
         int contentHeight = btn->fontMetrics.height();
 
         if (!btn->icon.isNull()) {
@@ -931,13 +983,32 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
         break;
     case ComboBox: {
         QStyleOptionComboBox *btn = qstyleoption_cast<QStyleOptionComboBox*>(m_styleoption);
-        int newWidth = qMax(width, btn->fontMetrics.width(btn->currentText));
+        int newWidth = qMax(width, btn->fontMetrics.boundingRect(btn->currentText).width());
         int newHeight = qMax(height, btn->fontMetrics.height());
         size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_ComboBox, m_styleoption, QSize(newWidth, newHeight)); }
         break;
-    case Tab:
-        size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_TabBarTab, m_styleoption, QSize(width,height));
+    case Tab: {
+        QStyleOptionTab *tab = qstyleoption_cast<QStyleOptionTab*>(m_styleoption);
+
+        int contentWidth = tab->fontMetrics.boundingRect(tab->text).width();
+        int contentHeight = tab->fontMetrics.height();
+
+        if (!tab->icon.isNull()) {
+            //+4 matches a hardcoded value in QStyle and acts as a margin between the icon and the text.
+            contentWidth += tab->iconSize.width() + 4;
+            contentHeight = qMax(contentHeight, tab->iconSize.height());
+        }
+
+        contentWidth += KQuickStyleItem::style()->pixelMetric(QStyle::PM_TabBarTabHSpace, tab);
+        contentHeight += KQuickStyleItem::style()->pixelMetric(QStyle::PM_TabBarTabVSpace, tab);
+
+        const int newWidth = qMax(width, contentWidth);
+        const int newHeight = qMax(height, contentHeight);
+
+        size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_TabBarTab, m_styleoption, QSize(newWidth, newHeight));
+
         break;
+    }
     case Slider:
         size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_Slider, m_styleoption, QSize(width,height));
         break;
@@ -967,7 +1038,7 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
     case GroupBox: {
             QStyleOptionGroupBox *box = qstyleoption_cast<QStyleOptionGroupBox*>(m_styleoption);
             QFontMetrics metrics(box->fontMetrics);
-            int baseWidth = metrics.width(box->text) + metrics.width(QLatin1Char(' '));
+            int baseWidth = metrics.boundingRect(box->text + QLatin1Char(' ')).width();
             int baseHeight = metrics.height() + m_contentHeight;
             if (box->subControls & QStyle::SC_GroupBoxCheckBox) {
                 baseWidth += KQuickStyleItem::style()->pixelMetric(QStyle::PM_IndicatorWidth);
@@ -1289,6 +1360,10 @@ void KQuickStyleItem::setElementType(const QString &str)
     } else {
         m_itemType = Undefined;
     }
+    emit leftPaddingChanged();
+    emit rightPaddingChanged();
+    emit topPaddingChanged();
+    emit bottomPaddingChanged();
     updateSizeHint();
 }
 
@@ -1404,8 +1479,8 @@ void KQuickStyleItem::paint(QPainter *painter)
     case ItemRow :{
         QPixmap pixmap;
         // Only draw through style once
-        const QString pmKey = QLatin1Literal("itemrow") % QString::number(m_styleoption->state,16) % activeControl();
-        if (!QPixmapCache::find(pmKey, pixmap) || pixmap.width() < width() || height() != pixmap.height()) {
+        const QString pmKey = QLatin1String("itemrow") % QString::number(m_styleoption->state,16) % activeControl();
+        if (!QPixmapCache::find(pmKey, &pixmap) || pixmap.width() < width() || height() != pixmap.height()) {
             int newSize = width();
             pixmap = QPixmap(newSize, height());
             pixmap.fill(Qt::transparent);
@@ -1442,6 +1517,8 @@ void KQuickStyleItem::paint(QPainter *painter)
         }
         break;
     case Frame:
+        m_styleoption->state |= QStyle::State_Sunken;
+        m_styleoption->state &= ~QStyle::State_Raised;
         KQuickStyleItem::style()->drawControl(QStyle::CE_ShapedFrame, m_styleoption, painter);
         break;
     case FocusFrame:
